﻿using MongoDB.Bson.Serialization.Attributes;
using MongoDB.Driver;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace WebApplication1.Models
{
    public class Contact
    {
        [BsonId]
        public string Id { get; set; }
        public string Name { get; set; }
        public string Phone { get; set; }
        public string Email { get; set; }
        public DateTime LastModified { get; set; }
    }

    public interface IContactRepository
    {
        IEnumerable<Contact> GetAllContacts();
        Contact GetContact(string id);
        Contact AddContact(Contact item);
        bool RemoveContact(string id);
        bool UpdateContact(string id, Contact item);
    }

    public class ContactRepository : IContactRepository
    {
        MongoClient _server = null;
        IMongoDatabase _database = null;
        IMongoCollection<Contact> _contacts = null;

        public ContactRepository(string connection)
        {
            if (string.IsNullOrWhiteSpace(connection))
            {
                connection = "mongodb://localhost:27017";
            }

            _server = new MongoClient(connection);
            _database = _server.GetDatabase("Contacts");
            _contacts = _database.GetCollection<Contact>("contacts");

            // Reset database and add some default entries
            var filter = Builders<Contact>.Filter.Where(a => 1 == 1);
            _contacts.DeleteMany(filter);
            for (int index = 1; index < 5; index++)
            {
                Contact contact1 = new Contact
                {
                    Email = string.Format("test{0}@example.com", index),
                    Name = string.Format("test{0}", index),
                    Phone = string.Format("{0}{0}{0} {0}{0}{0} {0}{0}{0}{0}", index)
                };
                AddContact(contact1);
            }
        }

        public IEnumerable<Contact> GetAllContacts()
        {
            return _contacts.Find(a => 1 == 1).ToList();
        }

        public Contact GetContact(string id)
        {
            //IMongoQuery query = Query.EQ("_id", id);
            return _contacts.Find(a => a.Id == id).FirstOrDefault();
        }

        public Contact AddContact(Contact item)
        {
            item.Id = Guid.NewGuid().ToString();
            item.LastModified = DateTime.UtcNow;
            _contacts.InsertOne(item);
            return item;
        }

        public bool RemoveContact(string id)
        {
            var filter = Builders<Contact>.Filter.Where(a => a.Id == id);
            var result = _contacts.DeleteOne(filter);
            return result.DeletedCount == 1;
        }

        public bool UpdateContact(string id, Contact item)
        {
            var query = Builders<Contact>.Filter.Where(a => a.Id == id);
            item.LastModified = DateTime.UtcNow;
            var update = Builders<Contact>.Update
                .Set("Email", item.Email)
                .Set("LastModified", DateTime.UtcNow)
                .Set("Name", item.Name)
                .Set("Phone", item.Phone);
            var result = _contacts.UpdateOne(query, update);
            return result.ModifiedCount > 0;
        }
    }
}